/// <reference types="node" />
import { EventEmitter } from "ws";
interface ClientEvents {
    'donation': (event: string, donation: Function) => void;
    'connect': (event: string, listener: Function) => void;
}
export declare interface Client {
    on<U extends keyof ClientEvents>(event: U, listener: ClientEvents[U]): this;
}
export declare class Client extends EventEmitter {
    username: string;
    streamKey: `trstream-${string}`;
    userId: string | undefined;
    /**
     * Client Class
     * @param pageID should be the trakteer page id (NOT Username). check pageID in https://trakteer.id/manage/my-page/settings
     * @param streamKey should be `trstream-xxx`. check key in https://trakteer.id/manage/stream-settings
     */
    constructor(pageID: string, streamKey: `trstream-${string}`);
    /**
     * please use this before using any code. to initialize the client
     */
    start(): Promise<void>;
    on2(event: "donation" | "open", cb: CallableFunction): void;
    /**
     *
     * @param cb This will triggered when The client is ready.
     * @deprecated This will removed on first beta, please use `client.on("open")` instead.
     */
    onOpen(cb: CallableFunction): void;
    /**
     *
     * @param cb This will triggered when donation is detected
     * @deprecated This will removed on first beta, please use `client.on("donation")` instead.
     */
    onDonation(cb: CallableFunction): void;
}
export {};
